@@ -1,4 +1,4 @@
-/** cmark binding for R by Jeroen Ooms (2016)
+/** cmark binding for R by Jeroen Ooms (2021)
  *  Adapted from CLI in main.c
  *  NOTE: man(3) states that cmark assumes UTF-8 for both input and output
  */
@@ -40,38 +40,46 @@ static char* print_document(cmark_node *document, writer_format writer, int opti
   }
 }
 
-SEXP R_render_markdown(SEXP text, SEXP format, SEXP sourcepos, SEXP hardbreaks, SEXP smart, SEXP normalize, SEXP width, SEXP extensions) {
+SEXP R_render_markdown(SEXP text, SEXP format, SEXP sourcepos, SEXP hardbreaks, SEXP smart,
+                       SEXP normalize, SEXP footnotes, SEXP width, SEXP extensions) {
 
   /* input validation */
-  if(!isString(text))
-    error("Argument 'text' must be string.");
-  if(!isInteger(format))
-    error("Argument 'format' must be integer.");
-  if(!isLogical(sourcepos))
-    error("Argument 'sourcepos' must be logical.");
-  if(!isLogical(hardbreaks))
-    error("Argument 'hardbreaks' must be logical.");
-  if(!isLogical(smart))
-    error("Argument 'smart' must be logical.");
-  if(!isLogical(normalize))
-    error("Argument 'normalize' must be logical.");
-  if(!isInteger(width))
-    error("Argument 'width' must be integer.");
+  if(!Rf_isString(text))
+    Rf_error("Argument 'text' must be string.");
+  if(!Rf_isInteger(format))
+    Rf_error("Argument 'format' must be integer.");
+  if(!Rf_isLogical(sourcepos))
+    Rf_error("Argument 'sourcepos' must be logical.");
+  if(!Rf_isLogical(hardbreaks))
+    Rf_error("Argument 'hardbreaks' must be logical.");
+  if(!Rf_isLogical(smart))
+    Rf_error("Argument 'smart' must be logical.");
+  if(!Rf_isLogical(normalize))
+    Rf_error("Argument 'normalize' must be logical.");
+  if(!Rf_isLogical(footnotes))
+    Rf_error("Argument 'footnotes' must be logical.");
+  if(!Rf_isInteger(width))
+    Rf_error("Argument 'width' must be integer.");
 
   /* combine options */
   int options = CMARK_OPT_DEFAULT;
-  options += asLogical(sourcepos) * CMARK_OPT_SOURCEPOS;
-  options += asLogical(hardbreaks) * CMARK_OPT_HARDBREAKS;
-  options += asLogical(smart) * CMARK_OPT_SMART;
-  options += asLogical(normalize) * CMARK_OPT_NORMALIZE;
+  options += Rf_asLogical(sourcepos) * CMARK_OPT_SOURCEPOS;
+  options += Rf_asLogical(hardbreaks) * CMARK_OPT_HARDBREAKS;
+  options += Rf_asLogical(smart) * CMARK_OPT_SMART;
+  options += Rf_asLogical(normalize) * CMARK_OPT_NORMALIZE;
+  options += Rf_asLogical(footnotes) * CMARK_OPT_FOOTNOTES;
+
+  /* Prevent filtering embedded resources: https://github.com/github/cmark-gfm#security */
+  options += CMARK_OPT_UNSAFE;
 
   /* parse input */
   SEXP input = STRING_ELT(text, 0);
   cmark_parser *parser = cmark_parser_new(options);
   for(int i = 0; i < Rf_length(extensions); i++){
-    cmark_syntax_extension *syntax_extension = cmark_find_syntax_extension(CHAR(STRING_ELT(extensions, i)));
+    const char *extname = CHAR(STRING_ELT(extensions, i));
+    cmark_syntax_extension *syntax_extension = cmark_find_syntax_extension(extname);
     if(!syntax_extension)
-      Rf_error("Failed to find load 'table' extension");
+      Rf_error("Failed to load extension '%s'", extname);
     cmark_parser_attach_syntax_extension(parser, syntax_extension);
   }
   cmark_parser_feed(parser, CHAR(input), LENGTH(input));
@@ -79,11 +87,11 @@ SEXP R_render_markdown(SEXP text, SEXP format, SEXP sourcepos, SEXP hardbreaks,
   cmark_parser_free(parser);
 
   /* render output format */
-  char *output = print_document(doc, asInteger(format), options, asInteger(width));
+  char *output = print_document(doc, Rf_asInteger(format), options, Rf_asInteger(width));
   cmark_node_free(doc);
 
   /* cmark always returns UTF8 output */
-  SEXP res = PROTECT(allocVector(STRSXP, 1));
+  SEXP res = PROTECT(Rf_allocVector(STRSXP, 1));
   SET_STRING_ELT(res, 0, Rf_mkCharCE(output, CE_UTF8));
   UNPROTECT(1);
   free(output);
